==> nono-0.6.3.tar.gz: nono-0.6.3/vm/xpbus.h <==
//
// nono
// Copyright (C) 2022 nono project
// Licensed under nono-license.txt
//

//
// XP プロセッサの物理バス
//

#pragma once

#include "mainbus.h"
#include <array>

class SubRAM2Device;
class XPRAMDevice;

// XP 外部バス
class XPbusDevice : public IODevice
{
	using inherited = IODevice;
 public:
	XPbusDevice();
	~XPbusDevice() override;

	bool Init() override;
	void ResetHard(bool poweron) override;

	uint64 Read8(uint32 addr) override;
	uint64 Read16(uint32 addr) override;
	uint64 Read32(uint32 addr) override;
	uint64 Write8(uint32 addr, uint32 data) override;
	uint64 Write16(uint32 addr, uint32 data) override;
	uint64 Write32(uint32 addr, uint32 data) override;
	uint64 Peek8(uint32 addr) override;
	uint64 Poke8(uint32 addr, uint32 data) override;

	// RAM Control Register (RMCR) を設定/取得
	uint32 GetRMCR() const;
	void SetRMCR(uint32);
	// 内蔵 RAM の先頭番地を取得 (モニタ用)
	uint32 GetXPRAMAddr() const { return ram_addr; }

 private:
	inline IODevice *SearchDevice(uint32 addr) const;

	// 3port RAM (非共有部)
	std::unique_ptr<SubRAM2Device> pSubRAM2 /*{}*/;
	// XP 内蔵 512 バイト RAM
	std::unique_ptr<XPRAMDevice> pXPRAM /*{}*/;

	// XP 内蔵 512 バイト RAM の位置
	uint32 ram_addr {};

	IODevice *subram {};	// 共有 SRAM
	IODevice *nopio {};
};

// XP 内蔵 RAM
class XPRAMDevice : public IODevice
{
	using inherited = IODevice;
 public:
	XPRAMDevice();
	~XPRAMDevice() override;

	uint64 Read8(uint32 addr) override;
	uint64 Read16(uint32 addr) override;
	uint64 Read32(uint32 addr) override;
	uint64 Write8(uint32 addr, uint32 data) override;
	uint64 Write16(uint32 addr, uint32 data) override;
	uint64 Write32(uint32 addr, uint32 data) override;
	uint64 Peek8(uint32 addr) override;
	uint64 Poke8(uint32 addr, uint32 data) override;

 private:
	std::array<uint8, 512> ram {};
};

static inline XPbusDevice *GetXPbusDevice() {
	return Object::GetObject<XPbusDevice>(OBJ_XPBUS);
}

==> nono-0.7.2.tar.gz: nono-0.7.2/vm/xpbus.h <==
//
// nono
// Copyright (C) 2022 nono project
// Licensed under nono-license.txt
//

//
// XP プロセッサの物理バス
//

#pragma once

#include "mainbus.h"
#include <array>

class SubRAM2Device;
class XPRAMDevice;

// XP 外部バス
class XPbusDevice : public IODevice
{
	using inherited = IODevice;
 public:
	XPbusDevice();
	~XPbusDevice() override;

	bool Init() override;
	void ResetHard(bool poweron) override;

	busdata Read8(uint32 addr) override;
	busdata Read16(uint32 addr) override;
	busdata Read32(uint32 addr) override;
	busdata Write8(uint32 addr, uint32 data) override;
	busdata Write16(uint32 addr, uint32 data) override;
	busdata Write32(uint32 addr, uint32 data) override;
	busdata Peek8(uint32 addr) override;
	bool Poke8(uint32 addr, uint32 data) override;

	// RAM Control Register (RMCR) を設定/取得
	uint32 GetRMCR() const;
	void SetRMCR(uint32);
	// 内蔵 RAM の先頭番地を取得 (モニタ用)
	uint32 GetXPRAMAddr() const { return ram_addr; }

 private:
	inline IODevice *SearchDevice(uint32 addr) const;

	// 3port RAM (非共有部)
	std::unique_ptr<SubRAM2Device> pSubRAM2 /*{}*/;
	// XP 内蔵 512 バイト RAM
	std::unique_ptr<XPRAMDevice> pXPRAM /*{}*/;

	// XP 内蔵 512 バイト RAM の位置
	uint32 ram_addr {};

	IODevice *subram {};	// 共有 SRAM
	IODevice *nopio {};
};

// XP 内蔵 RAM
class XPRAMDevice : public IODevice
{
	using inherited = IODevice;
 public:
	XPRAMDevice();
	~XPRAMDevice() override;

	busdata Read8(uint32 addr) override;
	busdata Read16(uint32 addr) override;
	busdata Read32(uint32 addr) override;
	busdata Write8(uint32 addr, uint32 data) override;
	busdata Write16(uint32 addr, uint32 data) override;
	busdata Write32(uint32 addr, uint32 data) override;
	busdata Peek8(uint32 addr) override;
	bool Poke8(uint32 addr, uint32 data) override;

 private:
	std::array<uint8, 512> ram {};
};

static inline XPbusDevice *GetXPbusDevice() {
	return Object::GetObject<XPbusDevice>(OBJ_XPBUS);
}
